function ch = moveg(ch, pnode, flag)
%   CHAN = MOVEG(CHAN, PNODE, FLAG)
%   moves ghost (CHAN) to next position.
%   PNODE is pacman's node property (we need to know where he is).
%   FLAG relates to the ghost's state
%       1: normal movement (default)
%       2: chillin in ghost house
%       3: moving out of ghost house

if strcmp(get(ch, 'type'),'pacman')
    error('MOVEG is for ghosts, blinking ghosts, and eyes, use MOVEP for pacman.')
end

if nargin==2
    flag=1;
elseif nargin == 1
    error('Where''s Pacman??  I don''t know!  Give me his node property damnit!');
end

%extract needed data from character
type = get(ch, 'type');
color = get(ch, 'color');
currentdir = get(ch, 'currentdir');
direction = get(ch, 'direction');
animation = get(ch, 'animation');
node = get(ch, 'node');
XData = get(ch.graphics, 'XData');
YData = get(ch.graphics, 'YData');

%get level and node dimension
lev = getappdata(gca, 'level');         %get level matrix
nodedim = get(lev, 'nodedim');

%allow ghosts to loop around level.
if node(1)>0 & node(1)<(28*nodedim)
    %get node and position
    [n, xpos, ypos] = getnodexy(lev,node(1),node(2));   %get node
    allow = get(n, 'allow');    %find where he can turn
else
    allow = [1 4];
    xpos = 0;
    ypos = 15;
end
if (xpos>=28 & ypos==15) | (xpos<=1 & ypos==15)
    allow = [allow  1 4];
end

%switch statement for different ghost types
switch type
    
    case 'ghost'
        
        %grab data
        data = getappdata(gca, 'ghost');
        eyes = getappdata(gca, 'eyes');
        
        switch flag
            case 1      %normal movement
                
                gdirections = {'right', 'down', 'up', 'left'};
                directions = {'left', 'up', 'down', 'right'};
                
                %get possible directions for ghost (can't turn around)
                allow = intersect(find(~strcmp(currentdir,gdirections)),allow);
                
                %change directions (possibly) if on node
                if animation ==1
                    %set probilities that different colors will turn
                    %correctly
                    switch color
                        case 'red'
                            prob = 0.75;
                        case 'pink'
                            prob = 0.65;
                        case 'cyan'
                            prob = 0.5;
                        case 'orange'
                            prob = 0.65;
                    end
                    %find the 'best direction' to go
                    if abs(node(1)-pnode(1))>abs(node(2)-pnode(2))
                        if (node(1)-pnode(1))>0
                            best1 = 1;
                        else
                            best1 = 4;
                        end
                    else
                        if (node(2)-pnode(2))>0
                            best1 = 3;
                        else
                            best1 = 2;
                        end
                    end
                    %find the 'second best direction' to go
                    if abs(node(1)-pnode(1))>abs(node(2)-pnode(2))
                        if (node(2)-pnode(2))>0
                            best2 = 3;
                        else
                            best2 = 2;
                        end
                    else
                        if (node(1)-pnode(1))>0
                            best2 = 1;
                        else
                            best2 = 4;
                        end
                    end
                    %random element
                    if sum(best1==allow) & (rand(1) <= prob)
                        currentdir = directions{best1};
                        ch.currentdir = currentdir;
                    elseif sum(best2==allow) & (rand(1) <= prob)
                        currentdir = directions{best2};
                        ch.currentdir = currentdir;
                    else
                        currentdir = directions{allow(ceil(length(allow)*rand(1)))};
                        ch.currentdir = currentdir;
                    end
                end
                
                %move the little fella...
                animation = mod(animation,4)+1; %update animation step
                ch.animation = animation;
                S.type = '.';
                S.subs = color;
                cdata = subsref(data(mod(animation-1,2)+1),S);
                switch currentdir
                    case 'left'
                        set(ch.graphics, 'CData', cdata.pxl .* eyes.left.sockets + eyes.left.pxl,...
                                         'AlphaData', cdata.alpha,...
                                         'XData', XData-[0.25*nodedim 0.25*nodedim]);
                        node(1) = mean(XData-[0.25*nodedim 0.25*nodedim]);
                    case 'right'
                        set(ch.graphics, 'CData', cdata.pxl .* eyes.right.sockets + eyes.right.pxl,...
                                         'AlphaData', cdata.alpha,...
                                         'XData', XData+[0.25*nodedim 0.25*nodedim]);
                        node(1) = mean(XData+[0.25*nodedim 0.25*nodedim]);
                    case 'up'
                        set(ch.graphics, 'CData', cdata.pxl .* eyes.up.sockets + eyes.up.pxl,...
                                         'AlphaData', cdata.alpha,...
                                         'YData', YData+[0.25*nodedim 0.25*nodedim]);
                        node(2) = mean(YData+[0.25*nodedim 0.25*nodedim]);
                    case 'down'
                        set(ch.graphics, 'CData', cdata.pxl .* eyes.down.sockets + eyes.down.pxl,...
                                         'AlphaData', cdata.alpha,...
                                         'YData', YData-[0.25*nodedim 0.25*nodedim]);
                        node(2) = mean(YData-[0.25*nodedim 0.25*nodedim]);
                end
                
            case 2  %chillin in the house
                
                %change direction
                if ypos==16 & animation==4
                    currentdir = 'up';
                    ch.currentdir = 'up';
                    animation=2;
                elseif ypos==14 & animation==4
                    currentdir = 'down';
                    ch.currentdir = 'down';
                    animation=2;
                end
                
                %move 'im
                animation = mod(animation,4)+1; %update animation step
                ch.animation = animation;
                S.type = '.';
                S.subs = color;
                cdata = subsref(data(mod(animation-1,2)+1),S);
                switch currentdir
                    case 'up'
                        set(ch.graphics, 'CData', cdata.pxl .* eyes.up.sockets + eyes.up.pxl,...
                                         'AlphaData', cdata.alpha,...
                                         'YData', YData+[0.25*nodedim 0.25*nodedim]);
                        node(2) = mean(YData+[0.25*nodedim 0.25*nodedim]);
                    case 'down'
                        set(ch.graphics, 'CData', cdata.pxl .* eyes.down.sockets + eyes.down.pxl,...
                                         'AlphaData', cdata.alpha,...
                                         'YData', YData-[0.25*nodedim 0.25*nodedim]);
                        node(2) = mean(YData-[0.25*nodedim 0.25*nodedim]);
                end 
                
            case 3  %leave the house
                
                [n, xpos, ypos] = getnodexy(lev,node(1)+0.5*nodedim,node(2));   %get node
                
                %move to center
                if xpos<15
                    currentdir = 'right';
                elseif xpos>15
                    currentdir = 'left';
                end
                
                %move up
                if xpos==15 & animation==1
                    currentdir = 'up';
                end
                
                %stop moving up
                if ypos==12 & animation==1
                    if rand(1)<=0.5
                        currentdir = 'left';
                    else
                        currentdir = 'right';
                    end
                    animation = 3;
                    ch.animation = 3;
                    assignin('caller',inputname(3),1);
                end
                
                ch.currentdir = currentdir;
                
                %move the little fella...
                animation = mod(animation,4)+1; %update animation step
                ch.animation = animation;
                S.type = '.';
                S.subs = color;
                cdata = subsref(data(mod(animation-1,2)+1),S);
                switch currentdir
                    case 'left'
                        set(ch.graphics, 'CData', cdata.pxl .* eyes.left.sockets + eyes.left.pxl,...
                                         'AlphaData', cdata.alpha,...
                                         'XData', XData-[0.25*nodedim 0.25*nodedim]);
                        node(1) = mean(XData-[0.25*nodedim 0.25*nodedim]);
                    case 'right'
                        set(ch.graphics, 'CData', cdata.pxl .* eyes.right.sockets + eyes.right.pxl,...
                                         'AlphaData', cdata.alpha,...
                                         'XData', XData+[0.25*nodedim 0.25*nodedim]);
                        node(1) = mean(XData+[0.25*nodedim 0.25*nodedim]);
                    case 'up'
                        set(ch.graphics, 'CData', cdata.pxl .* eyes.up.sockets + eyes.up.pxl,...
                                         'AlphaData', cdata.alpha,...
                                         'YData', YData+[0.25*nodedim 0.25*nodedim]);
                        node(2) = mean(YData+[0.25*nodedim 0.25*nodedim]);
                    case 'down'
                        set(ch.graphics, 'CData', cdata.pxl .* eyes.down.sockets + eyes.down.pxl,...
                                         'AlphaData', cdata.alpha,...
                                         'YData', YData-[0.25*nodedim 0.25*nodedim]);
                        node(2) = mean(YData-[0.25*nodedim 0.25*nodedim]);
                end
                
        end
        
    case 'eyes'
        
        %grab data
        eyes = getappdata(gca, 'eyes');
        
        gdirections = {'right', 'down', 'up', 'left'};
        directions = {'left', 'up', 'down', 'right'};
        
        [n, xpos, ypos] = getnodexy(lev,node(1),node(2));   %get node
                
        if xpos<17 & xpos>12 & ypos>12 & ypos<17
            allow = [2 3];
        end
        
        %get possible directions for ghost (can't turn around)
        allow = intersect(find(~strcmp(currentdir,gdirections)),allow);

        %set target
        pnode = [84 117];
        
        %change directions (possibly) if on node
        if animation ==1
            %set probability
            prob = 1;
            
            %find the 'best direction' to go
            if abs(node(1)-pnode(1))>abs(node(2)-pnode(2))
                if (node(1)-pnode(1))>0
                    best1 = 1;
                else
                    best1 = 4;
                end
            else
                if (node(2)-pnode(2))>0
                    best1 = 3;
                else
                    best1 = 2;
                end
            end
            %find the 'second best direction' to go
            if abs(node(1)-pnode(1))>abs(node(2)-pnode(2))
                if (node(2)-pnode(2))>0
                    best2 = 3;
                else
                    best2 = 2;
                end
            else
                if (node(1)-pnode(1))>0
                    best2 = 1;
                else
                    best2 = 4;
                end
            end
            %random element
            if sum(best1==allow) & (rand(1) <= prob)
                currentdir = directions{best1};
                ch.currentdir = currentdir;
            elseif sum(best2==allow) & (rand(1) <= prob)
                currentdir = directions{best2};
                ch.currentdir = currentdir;
            else
                currentdir = directions{allow(ceil(length(allow)*rand(1)))};
                ch.currentdir = currentdir;
            end
        end
        
        
        if node==pnode
            currentdir='down';
            ch.currentdir='down';
            animation = 1;
            ch.animation = 1;
        end
        
        if animation==1 & ypos==15 & xpos>12 & xpos<17
            currentdir='up';
            ch.currentdir='up';
            assignin('caller',inputname(3),3);
            ch.type = 'ghost';
            switch color
                case 'red'
                    assignin('caller', 'bspeed', 1);
                case 'pink'
                    assignin('caller', 'pspeed', 1);
                case 'cyan'
                    assignin('caller', 'ispeed', 1);
                case 'orange'
                    assignin('caller', 'cspeed', 1);
            end
        end
                
        %move the little fella...
        animation = mod(animation,4)+1; %update animation step
        ch.animation = animation;
        switch currentdir
            case 'left'
                set(ch.graphics, 'CData', eyes.left.pxl,...
                                 'AlphaData', eyes.left.alpha,...
                                 'XData', XData-[0.25*nodedim 0.25*nodedim]);
                node(1) = mean(XData-[0.25*nodedim 0.25*nodedim]);
            case 'right'
                set(ch.graphics, 'CData', eyes.right.pxl,...
                                 'AlphaData', eyes.right.alpha,...
                                 'XData', XData+[0.25*nodedim 0.25*nodedim]);
                node(1) = mean(XData+[0.25*nodedim 0.25*nodedim]);
            case 'up'
                set(ch.graphics, 'CData', eyes.up.pxl,...
                                 'AlphaData', eyes.up.alpha,...
                                 'YData', YData+[0.25*nodedim 0.25*nodedim]);
                node(2) = mean(YData+[0.25*nodedim 0.25*nodedim]);
            case 'down'
                set(ch.graphics, 'CData', eyes.down.pxl,...
                                 'AlphaData', eyes.down.alpha,...
                                 'YData', YData-[0.25*nodedim 0.25*nodedim]);
                node(2) = mean(YData-[0.25*nodedim 0.25*nodedim]);
        end
        
    otherwise   %handles ghosts and blinking ghosts
        
        %grab data
        data = getappdata(gca, 'ghost');
        bghost = getappdata(gca, 'bghost');
        
        switch flag
            case 1      %normal movement
                
                gdirections = {'right', 'down', 'up', 'left'};
                directions = {'left', 'up', 'down', 'right'};
                
                %get possible directions for ghost (can't turn around)
                allow = intersect(find(~strcmp(currentdir,gdirections)),allow);
                
                %change directions (possibly) if on node
                if animation ==1
                    %set probilities that different colors will turn
                    %correctly
                    switch color
                        case 'red'
                            prob = 0.75;
                        case 'pink'
                            prob = 0.65;
                        case 'cyan'
                            prob = 0.3;
                        case 'orange'
                            prob = 0.5;
                    end
                    %find the 'best direction' to go....to get away
                    if abs(node(1)-pnode(1))>abs(node(2)-pnode(2))
                        if (node(2)-pnode(2))>0
                            best1 = 2;
                        else
                            best1 = 3;
                        end
                    else
                        if (node(1)-pnode(1))>0
                            best1 = 4;
                        else
                            best1 = 1;
                        end
                    end
                    %find the 'second best direction' to go.....to get away
                    if abs(node(1)-pnode(1))>abs(node(2)-pnode(2))
                        if (node(1)-pnode(1))>0
                            best2 = 4;
                        else
                            best2 = 1;
                        end
                    else
                        if (node(2)-pnode(2))>0
                            best2 = 2;
                        else
                            best2 = 3;
                        end
                    end
                    %random element
                    if sum(best1==allow) & (rand(1) <= prob)
                        currentdir = directions{best1};
                        ch.currentdir = currentdir;
                    elseif sum(best2==allow) & (rand(1) <= prob)
                        currentdir = directions{best2};
                        ch.currentdir = currentdir;
                    else
                        currentdir = directions{allow(ceil(length(allow)*rand(1)))};
                        ch.currentdir = currentdir;
                    end
                end
                
                %move the little fella...
                animation = mod(animation,4)+1; %update animation step
                ch.animation = animation;
                if animation < 3
                    cdata = data(mod(animation-1,2)+1).blue;
                    eyes = bghost.blue;
                elseif animation >=3 & strcmp(type,'bghost')
                    cdata = data(mod(animation-1,2)+1).blue;
                    eyes = bghost.blue;
                else
                    cdata = data(mod(animation-1,2)+1).white;
                    eyes = bghost.white;
                end
                
            case 2  %chillin in the house
                
                %change direction
                if ypos==16 & animation==4
                    currentdir = 'up';
                    ch.currentdir = 'up';
                    animation=2;
                elseif ypos==14 & animation==4
                    currentdir = 'down';
                    ch.currentdir = 'down';
                    animation=2;
                end
                
                %move 'im
                animation = mod(animation,4)+1; %update animation step
                ch.animation = animation;
                if animation < 3
                    cdata = data(mod(animation-1,2)+1).blue;
                    eyes = bghost.blue;
                elseif animation >=3 & strcmp(type,'bghost')
                    cdata = data(mod(animation-1,2)+1).blue;
                    eyes = bghost.blue;
                else
                    cdata = data(mod(animation-1,2)+1).white;
                    eyes = bghost.white;
                end
                
            case 3  %leave the house
                
                [n, xpos, ypos] = getnodexy(lev,node(1)+0.5*nodedim,node(2));   %get node
                
                %move to center
                if xpos<15
                    currentdir = 'right';
                elseif xpos>15
                    currentdir = 'left';
                end
                
                %move up
                if xpos==15 & animation==1
                    currentdir = 'up';
                end
                
                %stop moving up
                if ypos==12 & animation==1
                    if rand(1)<=0.5
                        currentdir = 'left';
                    else
                        currentdir = 'right';
                    end
                    animation = 3;
                    ch.animation = 3;
                    assignin('caller',inputname(3),1);
                end
                
                ch.currentdir = currentdir;
                
                %move the little fella...
                animation = mod(animation,4)+1; %update animation step
                ch.animation = animation;
                if animation < 3
                    cdata = data(mod(animation-1,2)+1).blue;
                    eyes = bghost.blue;
                elseif animation >=3 & strcmp(type,'bghost')
                    cdata = data(mod(animation-1,2)+1).blue;
                    eyes = bghost.blue;
                else
                    cdata = data(mod(animation-1,2)+1).white;
                    eyes = bghost.white;
                end
                
        end
        
        
        switch currentdir
            case 'left'
                set(ch.graphics, 'CData', cdata.pxl .* bghost.sockets + eyes.pxl,...
                                 'AlphaData', cdata.alpha,...
                                 'XData', XData-[0.25*nodedim 0.25*nodedim]);
                node(1) = mean(XData-[0.25*nodedim 0.25*nodedim]);
            case 'right'
                set(ch.graphics, 'CData', cdata.pxl .* bghost.sockets + eyes.pxl,...
                                 'AlphaData', cdata.alpha,...
                                 'XData', XData+[0.25*nodedim 0.25*nodedim]);
                node(1) = mean(XData+[0.25*nodedim 0.25*nodedim]);
            case 'up'
                set(ch.graphics, 'CData', cdata.pxl .* bghost.sockets + eyes.pxl,...
                                 'AlphaData', cdata.alpha,...
                                 'YData', YData+[0.25*nodedim 0.25*nodedim]);
                node(2) = mean(YData+[0.25*nodedim 0.25*nodedim]);
            case 'down'
                set(ch.graphics, 'CData', cdata.pxl .* bghost.sockets + eyes.pxl,...
                                 'AlphaData', cdata.alpha,...
                                 'YData', YData-[0.25*nodedim 0.25*nodedim]);
                node(2) = mean(YData-[0.25*nodedim 0.25*nodedim]);
        end
        
end
 


%sound command
switch animation
    case 1
        %sound(ch.sound{1},ch.sound{3});
    case 3
        %sound(ch.sound{2},ch.sound{3});
end

%loop ghost around
if (node(1) <= -0.5*nodedim) & strcmp(currentdir, 'left')
    node(1) = 28.5*nodedim;
    corn = corners(gca,node);
    set(ch.graphics, 'XData', corn{1});
elseif (node(1) >= 28.5*nodedim) & strcmp(currentdir, 'right')
    node(1) = -0.5*nodedim;
    corn = corners(gca,node);
    set(ch.graphics, 'XData', corn{1});
end    

ch.node = node;