#include "mex.h"
#include <math.h>

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
/* This function is designed to blackout a block of pixels
 * in an image data matrix, as specified by a range of rows 
 * and columns. 
 *
 * a sample call to this function:
 * blackout(cdata, rowStart, rowEnd, colStart, colEnd)
 *
 * where cdata is assumed to be an MxNx3 matrix of RGB
 * values for an image, and the remaining inputs specify 
 * the bounds of the array indicies to be blacked out.
 * rowStart, etc. are specified in the manner that Matlab
 * thinks of them, with (1,1,1) being the first entry.
 * 
 * Note about changing the cdata of an image without using
 * the image's set command. The data will change, but using
 * a drawnow statement will not refresh the image for some
 * reason. One workaround for this is to set the image 
 * visibility off and on. You can also cover the figure 
 * with another image or some other window to get it to 
 * redraw the modified pixels. 
 */
 
/* First declare the appropriate variables */
int i, j, k, rowStart, rowEnd, colStart, colEnd, rows, cols;
unsigned char *pArray;   /* pointer to input */

/* Assign the variables. Note that the C code really treats the
 * matrix as a 1-dimensional array, so we have to use all the 
 * information regarding the number of rows and columns, etc.
 */
pArray      = mxGetPr(prhs[0]);
rows        = mxGetM(prhs[0]);
cols        = mxGetN(prhs[0]);
cols        = cols/3;              /* the third dimension is assumed to be 3 */

rowStart    = mxGetScalar(prhs[1]);
rowEnd      = mxGetScalar(prhs[2]);
colStart    = mxGetScalar(prhs[3]);
colEnd      = mxGetScalar(prhs[4]);

for(i=0; i<3; i++)      /* assume the array is 3 pages long in the 3rd dimension */
    {
    for(j=rowStart-1; j<rowEnd; j++)
        {
        for(k=colStart-1; k<colEnd; k++)
            {
            pArray[k*rows + j + i*rows*cols] = 0;
            }
        }
    }
}
