function pacman
%PACMAN
%   MATLAB version of the Pacman arcade game. The game was originally
%   created by Namco. It has been recreated for MATLAB by Kai Bauerbach,
%   Sanjay Dastoor, Arthur Hebert,and Trent Russi, with advising from
%   Andrew Packard.
%   Copyright, the above mentioned, 2004.
%
%   As of June 2004, the creators can be reached at the following addresses:
%
%   Andrew Packard      pack@jagger.me.berkeley.edu
%   Kai Bauerbach       kaibauerbach@gmx.de
%   Sanjay Dastoor      sanjayd@berkeley.edu
%   Arthur Hebert       ahebert@berkeley.edu
%   Trent Russi         trussi@berkeley.edu
%
%   Andrew Packard's address is mostly likely to be current

% Add files from the methods directory to the 
% current search path. Make sure to compensate
% for the different directory references on
% windows versus *nix platorms.
if ispc
    newdir = '\methods';
else
    newdir = '/methods';
end
path(path, [pwd, newdir]);


close all;
quit=0;
endpause = 1;

while ~quit
    
    % set some runtime varables
    exitnow = 0;
    pausing = 1;
    lives = 3;
    extralives = 0;
    lifejustadded = 0;
    splashflag = 1;
    score = 0;
    levelnumber = 1;
    
    %Game while loop - Each run through is a new level
    while lives & ~quit
        
        l=level('levels/lvlone.lv');   % load the level from the file 
        nodes=get(l,'nodes');   % acces the level's nodes
        node=nodes(24,15);      % set the start-node
        node2=nodes(12,15);
        node3=nodes(16,13);
        node4=nodes(16,15);
        node5=nodes(16,17);
        
        
        ahan=drawlevel(l);      % draw the level and get the axes-handle
        set(get(ahan, 'parent'), 'keypressfcn', @keycallback,...
            'NumberTitle', 'off',...
            'Name', ['Pacman: Level ', num2str(levelnumber)],...
            'position', (get(gcf, 'position')+[0 0 0 50]),...
            'closerequestfcn', @quit_callback);
        %stext(73,190,['Player: ' name],'color',[1 1 0]);
        setappdata(ahan, 'level', l);
        setappdata(ahan, 'pallets', get(l,'pallets'));
        setappdata(ahan, 'score', score);
        
        %set graphical data
        setpacmandata(ahan);
        
        %set up score board
        set(ahan,'position',get(ahan,'position')+[0 50 0 0]);
        tmp = get(ahan, 'position');
        ahan2 = axes('parent',gcf,'units','pixels','position',[20 20 tmp(3) 50],'XTick',[],'YTick',[],'color',[0 0 0]);
        pos = get(ahan2, 'position');
        axes(ahan2)
        axis([0 pos(3) 0 pos(4)]);
        text(30,30,'Score:','color',[1 1 1],'FontWeight','Bold');
        hold on;
        sh = text(100,30,num2str(getappdata(ahan,'score')),'color',[1 1 1],'FontWeight','Bold','FontSize',15);
        text(200,30,'Lives:','color',[1 1 1],'FontWeight','Bold');
        data = getappdata(ahan,'pacman');
        axes(ahan2);
        
        % create the symbols for how many lives you have.
        % For simplicity, we'll assume that no one is going
        % to get more than five lives. If someone gets more 
        % than the five lives, the game will play accordingly
        % but the icons on the bottom of the screen will only 
        % display the first five lives (four icons).
        ih1 = image([250 270],[40 20],data(2).left.pxl, 'visible', 'off');
        ih2 = image([280 300],[40 20],data(2).left.pxl, 'visible', 'off');
        ih3 = image([310 330],[40 20],data(2).left.pxl, 'visible', 'off');
        ih4 = image([340 360],[40 20],data(2).left.pxl, 'visible', 'off');
        
        if lives >= 5
            set(ih1, 'visible', 'on');
            set(ih2, 'visible', 'on');
            set(ih3, 'visible', 'on');
            set(ih4, 'visible', 'on');
        elseif lives == 4
            set(ih1, 'visible', 'on');
            set(ih2, 'visible', 'on');
            set(ih3, 'visible', 'on');
        elseif lives == 3
            set(ih1, 'visible', 'on');
            set(ih2, 'visible', 'on');
        elseif lives==2
            set(ih1, 'visible', 'on');
        end
        
        axes(ahan);
        
        newlevel=1;
        
        if splashflag
            quit = splashScreen;
            %disp('splashscreen')
            if quit == 1
                endpause = 0;
            end
            splashflag=0;
            set(get(ahan,'parent'),'keypressfcn',@keycallback);
        end
        
        %Level while loop - Each run through is another life
        while getappdata(ahan,'pallets') & lives & ~quit
            
            b = character(ahan, 'ghost', node2, 'red'); %make blinky
            i = character(ahan, 'ghost', node3, 'cyan'); %make inky
            p = character(ahan, 'ghost', node4, 'pink'); %make pinky
            c = character(ahan, 'ghost', node5, 'orange'); %make clyde
            pac = character(ahan, 'pacman', node);    %create a 'pacman' character  
            
            
            %change blinky's animation step
            if rand(1)>=0.5
                currentdir = 'right';
            else
                currentdir = 'left';
            end
            set(b,'animation',3,'currentdir',currentdir);
            
            %set set movement times
            blasttime = clock;
            plasttime = clock;
            ilasttime = clock;
            clasttime = clock;
            paclasttime = clock;
            
            %set leave house time
            leavetime = clock;
            
            %initialize blueclock
            blueclock = clock;
            pointblue = 200;
            
            %set movement flags
            blinky = 1;
            pinky = 3;
            inky = 2;
            clyde = 2;
            
            %set speeds
            bspeed = 1;
            pspeed = 1;
            ispeed = 1;
            cspeed = 1;
            
            %display ready!
            th = text(73,82,'Ready!','color',[1 1 0],'FontSize',17,'FontWeight','Bold');
            
            %play intro sound;
            drawnow;
            if newlevel & ispc  % make sure it's a windows PC
                [y, fs] = wavread('sounds/openingsong.wav');
                wavplay(y,fs);
            else
                pause(2)
            end
            newlevel=0;
            
            %last minute housekeeping
            delete(th);
            pausing = 0;
            
            %Animation while loop - Each run through is another animation step
            while~exitnow & getappdata(ahan,'pallets')
                %let inky out of house
                if etime(clock,leavetime)>5 & (inky==2) & (i.animation==1)
                    inky = 3;
                end
                
                %let clyde out of house
                if etime(clock,leavetime)>10 & (clyde==2) & (c.animation==1)
                    clyde = 3;
                end
                
                %change to blinking blue
                if etime(clock,blueclock)>10
                    if strcmp(b.type,'bghost')
                        b.type = 'bbghost';
                    end
                    if strcmp(p.type,'bghost')
                        p.type = 'bbghost';
                    end
                    if strcmp(i.type,'bghost')
                        i.type = 'bbghost';
                    end
                    if strcmp(c.type,'bghost')
                        c.type = 'bbghost';
                    end
                end
                
                %if blinking blue, change back to normal
                if etime(clock,blueclock)>15
                    if strcmp(b.type,'bbghost')
                        b.type='ghost';
                        bspeed = 1;
                    end
                    if strcmp(p.type,'bbghost')
                        p.type='ghost';
                        pspeed = 1;
                    end
                    if strcmp(i.type,'bbghost')
                        i.type='ghost';
                        ispeed = 1;
                    end
                    if strcmp(c.type,'bbghost')
                        c.type='ghost';
                        cspeed = 1;
                    end
                end
                
                %move pacman
                if etime(clock,paclasttime)>0.03
                    pac = movep(pac);
                    paclasttime = clock;
                end
                
                %move blinky
                if etime(clock,blasttime)>bspeed*0.03
                    b = moveg(b,get(pac,'node'),blinky);
                    blasttime = clock;
                end
                
                %move pinky
                if etime(clock,plasttime)>pspeed*0.02
                    p = moveg(p,get(pac,'node'),pinky);
                    plasttime = clock;
                end
                
                %move inky
                if etime(clock,ilasttime)>ispeed*0.03
                    i = moveg(i,get(pac,'node'),inky);
                    ilasttime = clock;
                end
                
                %move clyde
                if etime(clock,clasttime)>cspeed*0.03
                    c = moveg(c,get(pac,'node'),clyde);
                    clasttime = clock;
                end
                
                %check for collisions
                [exitnow, b, p, i, c] = collide(pac, b, p, i, c);
                
                %pause
                toadd = 0;
                tmp = clock;
                while pausing & ~exitnow    %pause
                    drawnow;
                    toadd = etime(clock,tmp);
                end
                %add time accued to timers
                leavetime(6) = leavetime(6) + toadd;
                
                %update score
                set(sh,'string',getappdata(ahan,'score'));
                
                % award an extra life for every 10,000 points
                if getappdata(ahan,'score')/10000 >= (extralives + 1) & getappdata(ahan, 'score')/10000 < extralives + 2
                    extralives = extralives + 1;
                    lives = lives + 1;
                    lifejustadded = 1;
                    disp('Extra Life!')
                end
                
                if lifejustadded
                    if lives >= 5
                        set(ih1, 'visible', 'on');
                        set(ih2, 'visible', 'on');
                        set(ih3, 'visible', 'on');
                        set(ih4, 'visible', 'on');
                    elseif lives == 4
                        set(ih1, 'visible', 'on');
                        set(ih2, 'visible', 'on');
                        set(ih3, 'visible', 'on');
                    elseif lives == 3
                        set(ih1, 'visible', 'on');
                        set(ih2, 'visible', 'on');
                    elseif lives==2
                        set(ih1, 'visible', 'on');
                    end
                    lifejustadded = 0;
                end
                
                drawnow;
                
                axes(ahan);
                
            end
            
            if getappdata(ahan,'pallets')
                %kill pacman
                [y,fs]=wavread('sounds/pacmandies.wav');
                pause(0.5);
                set(b,'visible','off');
                set(p,'visible','off');
                set(i,'visible','off');
                set(c,'visible','off');
                pacmandies = getappdata(ahan, 'pacmandies');
                if ispc
                    sound(y,fs);
                end
                set(pac.graphics,'alphadata',ones(14,14));
                for ii = [1:9]
                    set(pac.graphics,'cdata', pacmandies(ii).pxl);
                    pause(0.2);
                end
                
                %remove a life
                lives = lives-1;
                exitnow = 0;
                if lives == 4
                    set(ih4, 'visible', 'off');
                elseif lives == 3
                    set(ih3, 'visible', 'off');
                elseif lives==2
                    set(ih2, 'visible', 'off');
                elseif lives==1
                    set(ih1, 'visible', 'off');
                end
            end
            
            %delete characters
            delete(pac);
            delete(b);
            delete(p);
            delete(i);
            delete(c);
            
        end
        
        if getappdata(ahan,'pallets') == 0
            levelnumber = levelnumber + 1;
        end
        
        score = getappdata(ahan,'score');
        % delete(get(ahan, 'parent'));
    end
    
    th = text(58,82,'GAME    OVER','color',[1 0 0],'FontSize',17,'FontWeight','Bold');
    if endpause
        pause(2);
    end
    delete(th);
    delete(get(ahan, 'parent'));
end


close all;

%**************************************************************************
%**************************************************************************
%**************************************************************************
function keycallback(obj, eventData)

keypressed = double(get(obj, 'currentcharacter'));

if isempty(keypressed)
    keypressed=0;
end

switch keypressed
    case 28     % left arrow
        evalin('caller', 'pac.direction = ''left'';');
    case 29     % right arrow
        evalin('caller', 'pac.direction = ''right'';');
    case 30     % up arrow
        evalin('caller', 'pac.direction = ''up'';');
    case 31     % down arrow
        evalin('caller', 'pac.direction = ''down'';');
    case 9      % tab button
        pausing = evalin('caller', 'pausing;');
        pausing = mod(pausing+1,2);
        s = sprintf('pausing = %d;', pausing);
        evalin('caller', s);
    case {113, 81}     % 'q' or 'Q' button
        evalin('caller', 'exitnow = 1;');
        evalin('caller', 'lives = 1;');
        evalin('caller', 'quit = 1;');
        evalin('caller', 'endpause = 0;');
    otherwise
        ;
end

function quit_callback(obj, eventData)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function makes the program exit gracefully
% when the user tries to close the window.
evalin('caller', 'exitnow = 1;');
evalin('caller', 'lives = 1;');
evalin('caller', 'quit = 1;');
